package com.sun.org.apache.xerces.internal.jaxp.validation;

import com.sun.org.apache.xerces.internal.xni.grammars.XMLGrammarDescription;
import com.sun.org.apache.xerces.internal.util.XMLGrammarPoolImpl;
import com.sun.org.apache.xerces.internal.xni.parser.XMLConfigurationException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import com.sun.org.apache.xerces.internal.util.SAXMessageFormatter;
import com.sun.org.apache.xerces.internal.util.Status;
import com.sun.org.apache.xerces.internal.xni.grammars.XMLGrammarPool;
import com.sun.org.apache.xerces.internal.xni.grammars.Grammar;
import java.io.Reader;
import java.io.InputStream;
import org.xml.sax.InputSource;
import org.w3c.dom.Node;
import javax.xml.stream.XMLEventReader;
import java.io.IOException;
import org.xml.sax.SAXParseException;
import com.sun.org.apache.xerces.internal.xni.XNIException;
import org.xml.sax.SAXException;
import com.sun.org.apache.xerces.internal.util.SAXInputSource;
import com.sun.org.apache.xerces.internal.util.DOMInputSource;
import com.sun.org.apache.xerces.internal.util.StAXInputSource;
import javax.xml.transform.stax.StAXSource;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;
import com.sun.org.apache.xerces.internal.xni.parser.XMLInputSource;
import javax.xml.validation.Schema;
import javax.xml.transform.Source;
import com.sun.org.apache.xerces.internal.impl.Constants;
import jdk.xml.internal.JdkXmlFeatures;
import com.sun.org.apache.xerces.internal.utils.XMLSecurityPropertyManager;
import com.sun.org.apache.xerces.internal.utils.XMLSecurityManager;
import com.sun.org.apache.xerces.internal.util.ErrorHandlerWrapper;
import com.sun.org.apache.xerces.internal.util.DOMEntityResolverWrapper;
import org.w3c.dom.ls.LSResourceResolver;
import org.xml.sax.ErrorHandler;
import com.sun.org.apache.xerces.internal.impl.xs.XMLSchemaLoader;
import javax.xml.validation.SchemaFactory;

public final class XMLSchemaFactory extends SchemaFactory
{
    private static final String JAXP_SOURCE_FEATURE_PREFIX = "http://javax.xml.transform";
    private static final String SCHEMA_FULL_CHECKING = "http://apache.org/xml/features/validation/schema-full-checking";
    private static final String USE_GRAMMAR_POOL_ONLY = "http://apache.org/xml/features/internal/validation/schema/use-grammar-pool-only";
    private static final String XMLGRAMMAR_POOL = "http://apache.org/xml/properties/internal/grammar-pool";
    private static final String SECURITY_MANAGER = "http://apache.org/xml/properties/security-manager";
    private static final String XML_SECURITY_PROPERTY_MANAGER = "http://www.oracle.com/xml/jaxp/properties/xmlSecurityPropertyManager";
    private final XMLSchemaLoader fXMLSchemaLoader;
    private ErrorHandler fErrorHandler;
    private LSResourceResolver fLSResourceResolver;
    private final DOMEntityResolverWrapper fDOMEntityResolverWrapper;
    private final ErrorHandlerWrapper fErrorHandlerWrapper;
    private XMLSecurityManager fSecurityManager;
    private XMLSecurityPropertyManager fSecurityPropertyMgr;
    private final XMLSchemaFactory.XMLGrammarPoolWrapper fXMLGrammarPoolWrapper;
    private boolean fUseGrammarPoolOnly;
    private final JdkXmlFeatures fXmlFeatures;
    private final boolean fOverrideDefaultParser;
    
    public XMLSchemaFactory() {
        this.fXMLSchemaLoader = new XMLSchemaLoader();
        this.fErrorHandlerWrapper = new ErrorHandlerWrapper(DraconianErrorHandler.getInstance());
        this.fDOMEntityResolverWrapper = new DOMEntityResolverWrapper();
        this.fXMLGrammarPoolWrapper = new XMLSchemaFactory.XMLGrammarPoolWrapper();
        this.fXMLSchemaLoader.setFeature("http://apache.org/xml/features/validation/schema-full-checking", true);
        this.fXMLSchemaLoader.setProperty("http://apache.org/xml/properties/internal/grammar-pool", this.fXMLGrammarPoolWrapper);
        this.fXMLSchemaLoader.setEntityResolver(this.fDOMEntityResolverWrapper);
        this.fXMLSchemaLoader.setErrorHandler(this.fErrorHandlerWrapper);
        this.fUseGrammarPoolOnly = true;
        this.fSecurityManager = new XMLSecurityManager(true);
        this.fXMLSchemaLoader.setProperty("http://apache.org/xml/properties/security-manager", this.fSecurityManager);
        this.fSecurityPropertyMgr = new XMLSecurityPropertyManager();
        this.fXMLSchemaLoader.setProperty("http://www.oracle.com/xml/jaxp/properties/xmlSecurityPropertyManager", this.fSecurityPropertyMgr);
        this.fXmlFeatures = new JdkXmlFeatures(this.fSecurityManager.isSecureProcessing());
        this.fOverrideDefaultParser = this.fXmlFeatures.getFeature(JdkXmlFeatures.XmlFeature.JDK_OVERRIDE_PARSER);
        this.fXMLSchemaLoader.setFeature("jdk.xml.overrideDefaultParser", this.fOverrideDefaultParser);
    }
    
    @Override
    public boolean isSchemaLanguageSupported(String schemaLanguage) {
        if (schemaLanguage == null) {
            throw new NullPointerException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "SchemaLanguageNull", (Object[])null));
        }
        if (schemaLanguage.length() == 0) {
            throw new IllegalArgumentException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "SchemaLanguageLengthZero", (Object[])null));
        }
        return schemaLanguage.equals("http://www.w3.org/2001/XMLSchema") || schemaLanguage.equals(Constants.W3C_XML_SCHEMA10_NS_URI);
    }
    
    @Override
    public LSResourceResolver getResourceResolver() {
        return this.fLSResourceResolver;
    }
    
    @Override
    public void setResourceResolver(LSResourceResolver resourceResolver) {
        this.fLSResourceResolver = resourceResolver;
        this.fDOMEntityResolverWrapper.setEntityResolver(resourceResolver);
        this.fXMLSchemaLoader.setEntityResolver(this.fDOMEntityResolverWrapper);
    }
    
    @Override
    public ErrorHandler getErrorHandler() {
        return this.fErrorHandler;
    }
    
    @Override
    public void setErrorHandler(ErrorHandler errorHandler) {
        this.fErrorHandler = errorHandler;
        this.fErrorHandlerWrapper.setErrorHandler((errorHandler != null) ? errorHandler : DraconianErrorHandler.getInstance());
        this.fXMLSchemaLoader.setErrorHandler(this.fErrorHandlerWrapper);
    }
    
    @Override
    public Schema newSchema(Source[] schemas) throws SAXException {
        XMLSchemaFactory.XMLGrammarPoolImplExtension pool = new XMLSchemaFactory.XMLGrammarPoolImplExtension();
        this.fXMLGrammarPoolWrapper.setGrammarPool(pool);
        XMLInputSource[] xmlInputSources = new XMLInputSource[schemas.length];
        for (int i = 0; i < schemas.length; ++i) {
            Source source = schemas[i];
            if ((source instanceof StreamSource)) {
                StreamSource streamSource = (StreamSource)source;
                String publicId = streamSource.getPublicId();
                String systemId = streamSource.getSystemId();
                InputStream inputStream = streamSource.getInputStream();
                Reader reader = streamSource.getReader();
                XMLInputSource xmlInputSource = new XMLInputSource(publicId, systemId, null);
                xmlInputSource.setByteStream(inputStream);
                xmlInputSource.setCharacterStream(reader);
                xmlInputSources[i] = xmlInputSource;
            }
            else if ((source instanceof SAXSource)) {
                SAXSource saxSource = (SAXSource)source;
                InputSource inputSource = saxSource.getInputSource();
                if (inputSource == null) {
                    throw new SAXException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "SAXSourceNullInputSource", (Object[])null));
                }
                xmlInputSources[i] = new SAXInputSource(saxSource.getXMLReader(), inputSource);
            }
            else if ((source instanceof DOMSource)) {
                DOMSource domSource = (DOMSource)source;
                Node node = domSource.getNode();
                String systemID = domSource.getSystemId();
                xmlInputSources[i] = new DOMInputSource(node, systemID);
            }
            else if ((source instanceof StAXSource)) {
                StAXSource staxSource = (StAXSource)source;
                XMLEventReader eventReader = staxSource.getXMLEventReader();
                if (eventReader != null) {
                    xmlInputSources[i] = new StAXInputSource(eventReader);
                }
                else {
                    xmlInputSources[i] = new StAXInputSource(staxSource.getXMLStreamReader());
                }
            }
            else {
                if (source == null) {
                    throw new NullPointerException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "SchemaSourceArrayMemberNull", (Object[])null));
                }
                throw new IllegalArgumentException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "SchemaFactorySourceUnrecognized", new Object[] { source.getClass().getName() }));
            }
        }
        try {
            this.fXMLSchemaLoader.loadGrammar(xmlInputSources);
        }
        catch (XNIException e) {
            throw Util.toSAXException(e);
        }
        catch (IOException e2) {
            SAXParseException se = new SAXParseException(e2.getMessage(), null, e2);
            if (this.fErrorHandler != null) {
                this.fErrorHandler.error(se);
            }
            throw se;
        }
        this.fXMLGrammarPoolWrapper.setGrammarPool(null);
        int grammarCount = pool.getGrammarCount();
        AbstractXMLSchema schema = null;
        if (this.fUseGrammarPoolOnly) {
            if (grammarCount > 1) {
                schema = new XMLSchema(new ReadOnlyGrammarPool(pool));
            }
            else if (grammarCount == 1) {
                Grammar[] grammars = pool.retrieveInitialGrammarSet("http://www.w3.org/2001/XMLSchema");
                schema = new SimpleXMLSchema(grammars[0]);
            }
            else {
                schema = new EmptyXMLSchema();
            }
        }
        else {
            schema = new XMLSchema(new ReadOnlyGrammarPool(pool), false);
        }
        this.propagateFeatures(schema);
        this.propagateProperties(schema);
        return schema;
    }
    
    @Override
    public Schema newSchema() throws SAXException {
        AbstractXMLSchema schema = new WeakReferenceXMLSchema();
        this.propagateFeatures(schema);
        this.propagateProperties(schema);
        return schema;
    }
    
    public Schema newSchema(XMLGrammarPool pool) throws SAXException {
        AbstractXMLSchema schema = this.fUseGrammarPoolOnly ? new XMLSchema(new ReadOnlyGrammarPool(pool)) : new XMLSchema(pool, false);
        this.propagateFeatures(schema);
        return schema;
    }
    
    @Override
    public boolean getFeature(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
        if (name == null) {
            throw new NullPointerException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "FeatureNameNull", (Object[])null));
        }
        if (name.startsWith("http://javax.xml.transform") && (name.equals("http://javax.xml.transform.stream.StreamSource/feature") || name.equals("http://javax.xml.transform.sax.SAXSource/feature") || name.equals("http://javax.xml.transform.dom.DOMSource/feature") || name.equals("http://javax.xml.transform.stax.StAXSource/feature"))) {
            return true;
        }
        if (name.equals("http://javax.xml.XMLConstants/feature/secure-processing")) {
            return this.fSecurityManager != null && this.fSecurityManager.isSecureProcessing();
        }
        if (name.equals("http://apache.org/xml/features/internal/validation/schema/use-grammar-pool-only")) {
            return this.fUseGrammarPoolOnly;
        }
        try {
            return this.fXMLSchemaLoader.getFeature(name);
        }
        catch (XMLConfigurationException e) {
            String identifier = e.getIdentifier();
            if (e.getType() == Status.NOT_RECOGNIZED) {
                throw new SAXNotRecognizedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "feature-not-recognized", new Object[] { identifier }));
            }
            throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "feature-not-supported", new Object[] { identifier }));
        }
    }
    
    @Override
    public Object getProperty(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
        if (name == null) {
            throw new NullPointerException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "ProperyNameNull", (Object[])null));
        }
        if (name.equals("http://apache.org/xml/properties/security-manager")) {
            return this.fSecurityManager;
        }
        if (name.equals("http://apache.org/xml/properties/internal/grammar-pool")) {
            throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "property-not-supported", new Object[] { name }));
        }
        int index = this.fXmlFeatures.getIndex(name);
        if (index > -1) {
            return Boolean.valueOf(this.fXmlFeatures.getFeature(index));
        }
        try {
            return this.fXMLSchemaLoader.getProperty(name);
        }
        catch (XMLConfigurationException e) {
            String identifier = e.getIdentifier();
            if (e.getType() == Status.NOT_RECOGNIZED) {
                throw new SAXNotRecognizedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "property-not-recognized", new Object[] { identifier }));
            }
            throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "property-not-supported", new Object[] { identifier }));
        }
    }
    
    @Override
    public void setFeature(String name, boolean value) throws SAXNotRecognizedException, SAXNotSupportedException {
        if (name == null) {
            throw new NullPointerException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "FeatureNameNull", (Object[])null));
        }
        if (name.startsWith("http://javax.xml.transform") && (name.equals("http://javax.xml.transform.stream.StreamSource/feature") || name.equals("http://javax.xml.transform.sax.SAXSource/feature") || name.equals("http://javax.xml.transform.dom.DOMSource/feature") || name.equals("http://javax.xml.transform.stax.StAXSource/feature"))) {
            throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "feature-read-only", new Object[] { name }));
        }
        if (name.equals("http://javax.xml.XMLConstants/feature/secure-processing")) {
            if (System.getSecurityManager() != null && !value) {
                throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(null, "jaxp-secureprocessing-feature", (Object[])null));
            }
            this.fSecurityManager.setSecureProcessing(value);
            if (value && Constants.IS_JDK8_OR_ABOVE) {
                this.fSecurityPropertyMgr.setValue(XMLSecurityPropertyManager.Property.ACCESS_EXTERNAL_DTD, XMLSecurityPropertyManager.State.FSP, "");
                this.fSecurityPropertyMgr.setValue(XMLSecurityPropertyManager.Property.ACCESS_EXTERNAL_SCHEMA, XMLSecurityPropertyManager.State.FSP, "");
            }
            this.fXMLSchemaLoader.setProperty("http://apache.org/xml/properties/security-manager", this.fSecurityManager);
            return;
        }
        else {
            if (name.equals("http://apache.org/xml/features/internal/validation/schema/use-grammar-pool-only")) {
                this.fUseGrammarPoolOnly = value;
                return;
            }
            if (name.equals("http://www.oracle.com/feature/use-service-mechanism") && System.getSecurityManager() != null) {
                return;
            }
            if (this.fXmlFeatures != null && this.fXmlFeatures.setFeature(name, JdkXmlFeatures.State.APIPROPERTY, Boolean.valueOf(value))) {
                if (name.equals("jdk.xml.overrideDefaultParser") || name.equals("http://www.oracle.com/feature/use-service-mechanism")) {
                    this.fXMLSchemaLoader.setFeature(name, value);
                }
                return;
            }
            else {
                try {
                    this.fXMLSchemaLoader.setFeature(name, value);
                }
                catch (XMLConfigurationException e) {
                    String identifier = e.getIdentifier();
                    if (e.getType() == Status.NOT_RECOGNIZED) {
                        throw new SAXNotRecognizedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "feature-not-recognized", new Object[] { identifier }));
                    }
                    throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "feature-not-supported", new Object[] { identifier }));
                }
                return;
            }
        }
    }
    
    @Override
    public void setProperty(String name, Object object) throws SAXNotRecognizedException, SAXNotSupportedException {
        if (name == null) {
            throw new NullPointerException(JAXPValidationMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "ProperyNameNull", (Object[])null));
        }
        if (name.equals("http://apache.org/xml/properties/security-manager")) {
            this.fSecurityManager = XMLSecurityManager.convert(object, this.fSecurityManager);
            this.fXMLSchemaLoader.setProperty("http://apache.org/xml/properties/security-manager", this.fSecurityManager);
            return;
        }
        if (name.equals("http://www.oracle.com/xml/jaxp/properties/xmlSecurityPropertyManager")) {
            if (object == null) {
                this.fSecurityPropertyMgr = new XMLSecurityPropertyManager();
            }
            else {
                this.fSecurityPropertyMgr = (XMLSecurityPropertyManager)object;
            }
            this.fXMLSchemaLoader.setProperty("http://www.oracle.com/xml/jaxp/properties/xmlSecurityPropertyManager", this.fSecurityPropertyMgr);
            return;
        }
        else {
            if (name.equals("http://apache.org/xml/properties/internal/grammar-pool")) {
                throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "property-not-supported", new Object[] { name }));
            }
            try {
                if ((this.fSecurityManager == null || !this.fSecurityManager.setLimit(name, XMLSecurityManager.State.APIPROPERTY, object)) && (this.fSecurityPropertyMgr == null || !this.fSecurityPropertyMgr.setValue(name, XMLSecurityPropertyManager.State.APIPROPERTY, object))) {
                    this.fXMLSchemaLoader.setProperty(name, object);
                }
            }
            catch (XMLConfigurationException e) {
                String identifier = e.getIdentifier();
                if (e.getType() == Status.NOT_RECOGNIZED) {
                    throw new SAXNotRecognizedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "property-not-recognized", new Object[] { identifier }));
                }
                throw new SAXNotSupportedException(SAXMessageFormatter.formatMessage(this.fXMLSchemaLoader.getLocale(), "property-not-supported", new Object[] { identifier }));
            }
            return;
        }
    }
    
    private void propagateFeatures(AbstractXMLSchema schema) {
        schema.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", this.fSecurityManager != null && this.fSecurityManager.isSecureProcessing());
        schema.setFeature("jdk.xml.overrideDefaultParser", this.fOverrideDefaultParser);
        String[] features = this.fXMLSchemaLoader.getRecognizedFeatures();
        for (int i = 0; i < features.length; ++i) {
            boolean state = this.fXMLSchemaLoader.getFeature(features[i]);
            schema.setFeature(features[i], state);
        }
    }
    
    private void propagateProperties(AbstractXMLSchema schema) {
        String[] properties = this.fXMLSchemaLoader.getRecognizedProperties();
        for (int i = 0; i < properties.length; ++i) {
            Object state = this.fXMLSchemaLoader.getProperty(properties[i]);
            schema.setProperty(properties[i], state);
        }
    }
    
    static class XMLGrammarPoolImplExtension extends XMLGrammarPoolImpl
    {
        public XMLGrammarPoolImplExtension() {
        }
        
        public XMLGrammarPoolImplExtension(int initialCapacity) {
            super(initialCapacity);
        }
        
        int getGrammarCount() {
            return this.fGrammarCount;
        }
    }
    
    static class XMLGrammarPoolWrapper implements XMLGrammarPool
    {
        private XMLGrammarPool fGrammarPool;
        
        @Override
        public Grammar[] retrieveInitialGrammarSet(String grammarType) {
            return this.fGrammarPool.retrieveInitialGrammarSet(grammarType);
        }
        
        @Override
        public void cacheGrammars(String grammarType, Grammar[] grammars) {
            this.fGrammarPool.cacheGrammars(grammarType, grammars);
        }
        
        @Override
        public Grammar retrieveGrammar(XMLGrammarDescription desc) {
            return this.fGrammarPool.retrieveGrammar(desc);
        }
        
        @Override
        public void lockPool() {
            this.fGrammarPool.lockPool();
        }
        
        @Override
        public void unlockPool() {
            this.fGrammarPool.unlockPool();
        }
        
        @Override
        public void clear() {
            this.fGrammarPool.clear();
        }
        
        void setGrammarPool(XMLGrammarPool grammarPool) {
            this.fGrammarPool = grammarPool;
        }
        
        XMLGrammarPool getGrammarPool() {
            return this.fGrammarPool;
        }
    }
}
